# -*- coding: utf-8 -*-

from __future__ import print_function
import hashlib
import logging
import xml.dom.minidom
from requests import Session

from .errors import (InvalidError, LoginError)

_LOGGER = logging.getLogger(__name__)


def get_text(nodelist):
    """Get the value from a text node."""
    value = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            value.append(node.data)
    return ''.join(value)


def get_node_value(node, name):
    """Get the value from a node."""
    return get_text(node.getElementsByTagName(name)[0].childNodes)


def bits(value):
    while value:
        bit = value & (~value+1)
        yield bit
        value ^= bit


class Fritzhome(object):
    """Fritzhome object to communicate with the device."""

    _sid = None
    _session = None

    def __init__(self, host, user, password):
        self._host = host
        self._user = user
        self._password = password
        self._session = Session()

    def _request(self, url, params=None, timeout=10):
        """Send a request with paramerters."""
        rsp = self._session.get(url, params=params, timeout=timeout)
        rsp.raise_for_status()
        return rsp.text.strip()

    def _login_request(self, username=None, secret=None):
        """Send a login request with paramerters."""
        url = 'http://' + self._host + '/login_sid.lua'
        params = {}
        if username:
            params['username'] = username
        if secret:
            params['response'] = secret

        plain = self._request(url, params)
        dom = xml.dom.minidom.parseString(plain)
        sid = get_text(dom.getElementsByTagName('SID')[0].childNodes)
        challenge = get_text(
            dom.getElementsByTagName('Challenge')[0].childNodes)

        return (sid, challenge)

    def _logout_request(self):
        """Send a logout request."""
        _LOGGER.info('logout')
        url = 'http://' + self._host + '/login_sid.lua'
        params = {
            'security:command/logout': '1',
            'sid': self._sid
        }

        self._request(url, params)

    @staticmethod
    def _create_login_secret(challenge, password):
        """Create a login secret."""
        to_hash = (challenge + '-' + password).encode('UTF-16LE')
        hashed = hashlib.md5(to_hash).hexdigest()
        return '{0}-{1}'.format(challenge, hashed)

    def _aha_request(self, cmd, ain=None, param=None, rf=str):
        """Send an AHA request."""
        url = 'http://' + self._host + '/webservices/homeautoswitch.lua'
        params = {
            'switchcmd': cmd,
            'sid': self._sid
        }
        if param:
            params['param'] = param
        if ain:
            params['ain'] = ain

        plain = self._request(url, params)
        if plain == 'inval':
            raise InvalidError

        if rf == bool:
            return bool(int(plain))
        return rf(plain)

    def login(self):
        """Login and get a valid session ID."""
        try:
            (sid, challenge) = self._login_request()
            if sid == '0000000000000000':
                secret = self._create_login_secret(challenge, self._password)
                (sid2, challenge) = self._login_request(username=self._user,
                                                        secret=secret)
                if sid2 == '0000000000000000':
                    _LOGGER.warning("login failed %s", sid2)
                    raise LoginError(self._user)
                self._sid = sid2
        except xml.parsers.expat.ExpatError:
            raise LoginError(self._user)

    def logout(self):
        """Logout."""
        self._logout_request()
        self._sid = None

    def get_device_elements(self):
        """Get the DOM elments for the device list."""
        plain = self._aha_request('getdevicelistinfos')
        dom = xml.dom.minidom.parseString(plain)
        _LOGGER.info(dom)
        return dom.getElementsByTagName("device")

    def get_device_element(self, ain):
        """Get the DOM elment for the specified device."""
        elements = self.get_device_elements()
        for element in elements:
            if element.getAttribute('identifier') == ain:
                return element
        return None

    def get_devices(self):
        """Get the list of all known devices."""
        devices = []
        for element in self.get_device_elements():
            device = FritzhomeDevice(self, node=element)
            devices.append(device)
        return devices

    def get_device_by_ain(self, ain):
        """Returns a device specified by the AIN."""
        devices = self.get_devices()
        for device in devices:
            if device.ain == ain:
                return device

    def get_device_present(self, ain):
        """Get the device presence."""
        return self._aha_request('getswitchpresent', ain=ain, rf=bool)

    def get_device_name(self, ain):
        """Get the device name."""
        return self._aha_request('getswitchname', ain=ain)

    def get_switch_state(self, ain):
        """Get the switch state."""
        return self._aha_request('getswitchstate', ain=ain, rf=bool)

    def set_switch_state_on(self, ain):
        """Set the switch to on state."""
        return self._aha_request('setswitchon', ain=ain, rf=bool)

    def set_switch_state_off(self, ain):
        """Set the switch to off state."""
        return self._aha_request('setswitchoff', ain=ain, rf=bool)

    def set_switch_state_toggle(self, ain):
        """Toglle the switch state."""
        return self._aha_request('setswitchtoggle', ain=ain, rf=bool)

    def get_switch_power(self, ain):
        """Get the switch power consumption."""
        return self._aha_request('getswitchpower', ain=ain, rf=int)

    def get_switch_energy(self, ain):
        """Get the switch energy."""
        return self._aha_request('getswitchenergy', ain=ain, rf=int)

    def get_temperature(self, ain):
        """Get the device temperature sensor value."""
        return self._aha_request('gettemperature', ain=ain, rf=float) / 10.0

    def _get_tempearture(self, ain, name):
        plain = self._aha_request(name, ain=ain, rf=float)
        return (plain - 16) / 2 + 8

    def get_target_temperature(self, ain):
        """Get the thermostate target temperature."""
        return self._get_tempearture(ain, 'gethkrtsoll')

    def set_target_temperature(self, ain, temperature):
        """Set the thermostate target temperature."""
        param = 16 + ((float(temperature) - 8) * 2)

        if param < min(range(16, 56)):
            param = 253
        elif param > max(range(16, 56)):
            param = 254
        self._aha_request('sethkrtsoll', ain=ain, param=int(param))

    def get_comfort_temperature(self, ain):
        """Get the thermostate comfort temperature."""
        return self._get_tempearture(ain, 'gethkrkomfort')

    def get_eco_temperature(self, ain):
        """Get the thermostate eco temperature."""
        return self._get_tempearture(ain, 'gethkrabsenk')

    def get_alert_state(self, ain):
        """Get the alert state."""
        device = self.get_device_by_ain(ain)
        return device.alert_state

    def get_device_statistic(self, ain):
        """Get device statistics."""
        plain = self._aha_request('getbasicdevicestats', ain=ain)
        return plain


class FritzhomeDevice(object):
    """The Fritzhome Device class."""
    ALARM_MASK = 0x010
    UNKNOWN_MASK = 0x020
    THERMOSTAT_MASK = 0x040
    POWER_METER_MASK = 0x080
    TEMPERATURE_MASK = 0x100
    SWITCH_MASK = 0x200
    DECT_REPEATER_MASK = 0x400
    MICROPHONE_UNIT = 0x800
    HANFUN_UNIT = 0x2000

    ain = None
    identifier = None
    manufacturer = None
    productname = None
    actual_temperature = None
    target_temperature = None
    eco_temperature = None
    comfort_temperature = None
    battery_level = None
    window_open = None
    summer_active = None
    holiday_active = None
    lock = None
    device_lock = None
    error_code = None
    battery_low = None
    switch_state = None
    switch_mode = None
    power = None
    energy = None
    voltage = None
    offset = None
    temperature = None
    alert_state = None

    def __init__(self, fritz=None, node=None):
        if fritz is not None:
            self._fritz = fritz
        if node is not None:
            self._update_from_node(node)

    @staticmethod
    def _get_temp_from_node(val, name):
        return float(get_node_value(val, name)) / 2

    def _update_from_node(self, node):
        _LOGGER.debug(node.toprettyxml())
        self.ain = node.getAttribute("identifier")
        self.identifier = node.getAttribute("id")
        self._functionsbitmask = int(node.getAttribute("functionbitmask"))
        self.fw_version = node.getAttribute("fwversion")
        self.manufacturer = node.getAttribute("manufacturer")
        self.productname = node.getAttribute("productname")

        self.name = get_node_value(node, 'name')
        self.present = bool(int(get_node_value(node, 'present')))

        if self.present is False:
            return

        for bit in bits(self._functionsbitmask):
            try:
                fct = {
                    self.ALARM_MASK: self._update_alarm_from_node,
                    self.POWER_METER_MASK: self._update_powermeter_from_node,
                    self.SWITCH_MASK: self._update_switch_from_node,
                    self.TEMPERATURE_MASK: self._update_temperature_from_node,
                    self.THERMOSTAT_MASK: self._update_hkr_from_node,
                }[bit]
                fct(node)
            except KeyError:
                pass

    def _update_hkr_from_node(self, node):
        val = node.getElementsByTagName('hkr')[0]

        try:
            self.actual_temperature = self._get_temp_from_node(val, 'tist')
        except ValueError:
            pass

        self.target_temperature = self._get_temp_from_node(val, 'tsoll')
        self.eco_temperature = self._get_temp_from_node(val, 'absenk')
        self.comfort_temperature = self._get_temp_from_node(val, 'komfort')

        # optional value
        try:
            self.device_lock = bool(int(get_node_value(val, 'devicelock')))
        except IndexError:
            pass

        try:
            self.lock = bool(int(get_node_value(val, 'lock')))
        except IndexError:
            pass

        try:
            self.error_code = int(get_node_value(val, 'errorcode'))
        except IndexError:
            pass

        try:
            self.battery_low = bool(int(get_node_value(val, 'batterylow')))
        except IndexError:
            pass

        try:
            self.battery_level = int(int(get_node_value(val, 'battery')))
        except IndexError:
            pass

        try:
            self.window_open = bool(int(get_node_value(val,
                                                       'windowopenactiv')))
        except IndexError:
            pass

        try:
            self.summer_active = bool(int(get_node_value(val, 'summeractive')))
        except IndexError:
            pass

        try:
            self.holiday_active = bool(int(get_node_value(val,
                                                          'holidayactive')))
        except IndexError:
            pass

    def _update_switch_from_node(self, node):
        val = node.getElementsByTagName('switch')[0]
        self.switch_state = bool(int(get_node_value(val, 'state')))
        self.switch_mode = get_node_value(val, 'mode')
        self.lock = bool(get_node_value(val, 'lock'))
        # optional value
        try:
            self.device_lock = bool(int(get_node_value(val, 'devicelock')))
        except IndexError:
            pass

    def _update_powermeter_from_node(self, node):
        val = node.getElementsByTagName('powermeter')[0]
        self.power = int(get_node_value(val, 'power'))
        self.energy = int(get_node_value(val, 'energy'))
        try:
            self.voltage = float(int(get_node_value(val, 'voltage')) / 1000)
        except IndexError:
            pass

    def _update_temperature_from_node(self, node):
        val = node.getElementsByTagName('temperature')[0]
        try:
            self.offset = int(get_node_value(val, 'offset')) / 10
        except ValueError:
            pass

        try:
            self.temperature = int(get_node_value(val, 'celsius')) / 10
        except ValueError:
            pass

    def _update_alarm_from_node(self, node):
        val = node.getElementsByTagName('alert')[0]
        try:
            self.alert_state = bool(int(get_node_value(val, 'state')))
        except IndexError:
            pass

    def __repr__(self):
        """Return a string."""
        return '{ain} {identifier} {manuf} {prod} {name}'.format(
            ain=self.ain,
            identifier=self.identifier,
            manuf=self.manufacturer,
            prod=self.productname,
            name=self.name)

    def update(self):
        """Update the device values."""
        node = self._fritz.get_device_element(self.ain)
        self._update_from_node(node)

    @property
    def has_alarm(self):
        """Check if the device has alarm function."""
        return bool(self._functionsbitmask & self.ALARM_MASK)

    @property
    def has_thermostat(self):
        """Check if the device has thermostat function."""
        return bool(self._functionsbitmask & self.THERMOSTAT_MASK)

    @property
    def has_powermeter(self):
        """Check if the device has powermeter function."""
        return bool(self._functionsbitmask & self.POWER_METER_MASK)

    @property
    def has_temperature_sensor(self):
        """Check if the device has temperature function."""
        return bool(self._functionsbitmask & self.TEMPERATURE_MASK)

    @property
    def has_switch(self):
        """Check if the device has switch function."""
        return bool(self._functionsbitmask & self.SWITCH_MASK)

    @property
    def has_repeater(self):
        """Check if the device has repeater function."""
        return bool(self._functionsbitmask & self.DECT_REPEATER_MASK)

    def get_present(self):
        """Check if the device is present."""
        return self._fritz.get_device_present(self.ain)

    def get_switch_state(self):
        """Get the switch state."""
        return self._fritz.get_switch_state(self.ain)

    def set_switch_state_on(self):
        """Set the switch state to on."""
        return self._fritz.set_switch_state_on(self.ain)

    def set_switch_state_off(self):
        """Set the switch state to off."""
        return self._fritz.set_switch_state_off(self.ain)

    def set_switch_state_toggle(self):
        """Toggle the switch state."""
        return self._fritz.set_switch_state_toggle(self.ain)

    def get_switch_power(self):
        """ the switch state."""
        return self._fritz.get_switch_power(self.ain)

    def get_switch_energy(self):
        """Get the switch energy."""
        return self._fritz.get_switch_energy(self.ain)

    def get_temperature(self):
        """Get the device temperature value."""
        return self._fritz.get_temperature(self.ain)

    def get_target_temperature(self):
        """Get the thermostate target temperature."""
        return self._fritz.get_target_temperature(self.ain)

    def set_target_temperature(self, temperature):
        """Set the thermostate target temperature."""
        return self._fritz.set_target_temperature(self.ain, temperature)

    def get_comfort_temperature(self):
        """Get the thermostate comfort temperature."""
        return self._fritz.get_comfort_temperature(self.ain)

    def get_eco_temperature(self):
        """Get the thermostate eco temperature."""
        return self._fritz.get_eco_temperature(self.ain)

    def get_hkr_state(self):
        """Get the thermostate state."""
        self.update()
        try:
            return {
                126.5: 'off',
                127.0: 'on',
                self.eco_temperature: 'eco',
                self.comfort_temperature: 'comfort'
            }[self.target_temperature]
        except KeyError:
            return 'manual'

    def set_hkr_state(self, state):
        """Set the state of the thermostat.

        Possible values for state are: 'on', 'off', 'comfort', 'eco'.
        """
        try:
            value = {
                'off': 0,
                'on': 100,
                'eco': self.eco_temperature,
                'comfort': self.comfort_temperature
            }[state]
        except KeyError:
            return

        self.set_target_temperature(value)



# -*- coding: utf-8 -*-

login_rsp_without_valid_sid = '<?xml version="1.0" encoding="utf-8"?><SessionInfo><SID>0000000000000000</SID><Challenge>44b750c0</Challenge><BlockTime>0</BlockTime><Rights></Rights></SessionInfo>'

login_rsp_with_valid_sid = '<?xml version="1.0" encoding="utf-8"?><SessionInfo><SID>0000000000000001</SID><Challenge>44b750c0</Challenge><BlockTime>0</BlockTime><Rights></Rights></SessionInfo>'

device_list_xml = """<devicelist version="1">
    <device identifier="08761 0000434" id="17" functionbitmask="896" fwversion="03.33" manufacturer="AVM" productname="FRITZ!DECT 200">
        <present>1</present>
        <name>Steckdose</name>
        <switch>
            <state>1</state>
            <mode>auto</mode>
            <lock>0</lock>
            <devicelock>0</devicelock>
        </switch>
        <powermeter>
            <power>0</power>
            <energy>707</energy>
        </powermeter>
        <temperature>
            <celsius>285</celsius>
            <offset>0</offset>
        </temperature>
    </device>
    <device identifier="08761 1048079" id="16" functionbitmask="1280"
    fwversion="03.44" manufacturer="AVM" productname="FRITZ!DECT Repeater 100">
        <present>1</present>
        <name>FRITZ!DECT Rep 100 #1</name>
        <temperature>
            <celsius>288</celsius>
            <offset>0</offset>
        </temperature>
    </device>
    <device identifier="11959 0171328" id="16" functionbitmask="320"
    fwversion="03.54" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Badezimmer</name>
        <temperature>
            <celsius>205</celsius>
            <offset>-15</offset>
        </temperature>
        <hkr>
            <tist>41</tist>
            <tsoll>36</tsoll>
            <absenk>36</absenk>
            <komfort>42</komfort>
            <lock>0</lock>
            <devicelock>0</devicelock>
            <errorcode>0</errorcode>
            <batterylow>0</batterylow>
            <nextchange>
                <endperiod>1508342400</endperiod>
                <tchange>42</tchange>
            </nextchange>
        </hkr>
    </device>
    <device functionbitmask="8208" fwversion="0.0" id="2000" 
     identifier="05333 0077045-1" manufacturer="0x0512" productname="HAN-FUN">
        <present>1</present>
        <name>Fenster</name>
        <etsiunitinfo>
            <etsideviceid>406</etsideviceid>
            <unittype>514</unittype>
            <interfaces>256</interfaces>
        </etsiunitinfo>
        <alert>
            <state>1</state>
        </alert>
    </device>
    <group identifier="65:3A:18-900" id="900" functionbitmask="512" fwversion="1.0" manufacturer="AVM" productname="">
        <present>1</present>
        <name>Gruppe</name>
        <switch>
            <state>1</state>
            <mode>auto</mode>
            <lock/>
            <devicelock/>
        </switch>
        <groupinfo>
            <masterdeviceid>0</masterdeviceid>
            <members>17</members>
        </groupinfo>
    </group>
</devicelist>"""

device_list_battery_ok_xml = """<devicelist version="1">
    <device identifier="11959 0171328" id="16" functionbitmask="320"
    fwversion="03.54" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Badezimmer</name>
        <temperature>
            <celsius>205</celsius>
            <offset>-15</offset>
        </temperature>
        <hkr>
            <tist>41</tist>
            <tsoll>36</tsoll>
            <absenk>36</absenk>
            <komfort>42</komfort>
            <lock>0</lock>
            <devicelock>0</devicelock>
            <errorcode>0</errorcode>
            <batterylow>0</batterylow>
            <nextchange>
                <endperiod>1508342400</endperiod>
                <tchange>42</tchange>
            </nextchange>
        </hkr>
    </device>
</devicelist>"""

device_list_battery_low_xml = """<devicelist version="1">
    <device identifier="11959 0171328" id="16" functionbitmask="320"
    fwversion="03.54" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Badezimmer</name>
        <temperature>
            <celsius>205</celsius>
            <offset>-15</offset>
        </temperature>
        <hkr>
            <tist>41</tist>
            <tsoll>36</tsoll>
            <absenk>36</absenk>
            <komfort>42</komfort>
            <lock>0</lock>
            <devicelock>0</devicelock>
            <errorcode>0</errorcode>
            <batterylow>1</batterylow>
            <nextchange>
                <endperiod>1508342400</endperiod>
                <tchange>42</tchange>
            </nextchange>
        </hkr>
    </device>
</devicelist>"""


device_not_present_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.54" id="18" identifier="11960 0089208" manufacturer="AVM" productname="Comet DECT">
        <present>0</present>
        <name>Kitchen</name>
        <temperature>
            <celsius/>
            <offset/>
        </temperature>
        <hkr>
            <tist/>
            <tsoll/>
            <absenk/>
            <komfort/>
            <lock/>
            <devicelock/>
            <errorcode>0</errorcode>
            <batterylow>0</batterylow>
            <nextchange>
                <endperiod>0</endperiod>
                <tchange>255</tchange>
            </nextchange>
        </hkr>
    </device>
</devicelist>"""

device_no_devicelock_element_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="896" fwversion="03.59" id="16" identifier="08761 0373130" manufacturer="AVM" productname="FRITZ!DECT 200">
            <present>1</present>
            <name>FRITZ!DECT 200 #1</name>
            <switch>
                    <state>1</state>
                    <mode>manuell</mode>
                    <lock>1</lock>
            </switch>
            <powermeter>
                    <power>114580</power>
                    <energy>87830</energy>
            </powermeter>
            <temperature>
                    <celsius>220</celsius>
                    <offset>0</offset>
            </temperature>
    </device>

</devicelist>"""

device_hkr_fw_03_50_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>45</tsoll>
            <absenk>32</absenk>
            <komfort>45</komfort>
        </hkr>
    </device>
</devicelist>"""

device_hkr_fw_03_54_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.54" id="18" identifier="23456" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Badezimmer</name>
        <temperature>
            <celsius>205</celsius>
            <offset>-15</offset>
        </temperature>
        <hkr>
            <tist>41</tist>
            <tsoll>36</tsoll>
            <absenk>36</absenk>
            <komfort>42</komfort>
            <lock>0</lock>
            <devicelock>0</devicelock>
            <errorcode>0</errorcode>
            <batterylow>0</batterylow>
            <nextchange>
                <endperiod>1508342400</endperiod>
                <tchange>42</tchange>
            </nextchange>
        </hkr>
    </device>
</devicelist>"""

device_hkr_no_temp_values_xml= """<device functionbitmask="320" fwversion="03.54" id="22" identifier="11960 0071472" manufacturer="AVM" productname="Comet DECT">
	<present>1</present>
	<name>Thermostat Wohnzimmer Seite</name>
	<temperature>
		<celsius/>
		<offset/>
	</temperature>
	<hkr>
		<tist>44</tist>
		<tsoll>42</tsoll>
		<absenk>36</absenk>
		<komfort>42</komfort>
		<lock>0</lock>
		<devicelock>0</devicelock>
		<errorcode>5</errorcode>
		<batterylow>0</batterylow>
		<nextchange>
			<endperiod>1519160400</endperiod>
			<tchange>36</tchange>
		</nextchange>
	</hkr>
</device>"""

device_alert_on_xml = """<device functionbitmask="8208" fwversion="0.0" id="2000" identifier="05333 0077045-1" manufacturer="0x0512" productname="HAN-FUN">
    <present>1</present>
    <name>Fenster</name>
    <etsiunitinfo>
        <etsideviceid>406</etsideviceid>
        <unittype>514</unittype>
        <interfaces>256</interfaces>
    </etsiunitinfo>
    <alert>
        <state>1</state>
    </alert>
</device>"""

device_alert_off_xml = """<device functionbitmask="8208" fwversion="0.0" id="2000" identifier="05333 0077045-2" manufacturer="0x0512" productname="HAN-FUN">
    <present>1</present>
    <name>Fenster</name>
    <etsiunitinfo>
        <etsideviceid>406</etsideviceid>
        <unittype>514</unittype>
        <interfaces>256</interfaces>
    </etsiunitinfo>
    <alert>
        <state>0</state>
    </alert>
</device>"""

device_alert_no_alertstate_xml = """<device functionbitmask="8208" fwversion="0.0" id="2000" identifier="05333 0077045-3" manufacturer="0x0512" productname="HAN-FUN">
    <present>1</present>
    <name>Fenster</name>
    <etsiunitinfo>
        <etsideviceid>406</etsideviceid>
        <unittype>514</unittype>
        <interfaces>256</interfaces>
    </etsiunitinfo>
    <alert></alert>
</device>"""

device_with_umlaut_in_name_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="896" fwversion="03.59" id="16" identifier="08761 0373130" manufacturer="AVM" productname="FRITZ!DECT 200">
            <present>1</present>
            <name>äöü</name>
            <switch>
                    <state>1</state>
                    <mode>manuell</mode>
                    <lock>1</lock>
            </switch>
            <powermeter>
                    <power>114580</power>
                    <energy>87830</energy>
            </powermeter>
            <temperature>
                    <celsius>220</celsius>
                    <offset>0</offset>
            </temperature>
    </device>

</devicelist>"""

device_hkr_state_on_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>254</tsoll>
            <absenk>32</absenk>
            <komfort>45</komfort>
        </hkr>
    </device>
</devicelist>"""

device_hkr_state_off_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>253</tsoll>
            <absenk>32</absenk>
            <komfort>45</komfort>
        </hkr>
    </device>
</devicelist>"""

device_hkr_state_eco_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>45</tsoll>
            <absenk>45</absenk>
            <komfort>48</komfort>
        </hkr>
    </device>
</devicelist>"""

device_hkr_state_comfort_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>48</tsoll>
            <absenk>45</absenk>
            <komfort>48</komfort>
        </hkr>
    </device>
</devicelist>"""

device_hkr_state_manual_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.50" id="17" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Comet DECT #2</name>
        <temperature>
            <celsius>230</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>46</tist>
            <tsoll>40</tsoll>
            <absenk>45</absenk>
            <komfort>48</komfort>
        </hkr>
    </device>
</devicelist>"""


device_hkr_fritzos_7_xml = """<?xml version="1.0" ?>
<devicelist version="1">
    <device functionbitmask="320" fwversion="03.54" id="22" identifier="12345" manufacturer="AVM" productname="Comet DECT">
        <present>1</present>
        <name>Thermostat Wohnzimmer Seite</name>
        <temperature>
            <celsius>205</celsius>
            <offset>-10</offset>
        </temperature>
        <hkr>
            <tist>41</tist>
            <tsoll>253</tsoll>
            <absenk>36</absenk>
            <komfort>42</komfort>
            <lock>1</lock>
            <devicelock>1</devicelock>
            <errorcode>0</errorcode>
            <batterylow>0</batterylow>
            <windowopenactiv>0</windowopenactiv>
            <battery>70</battery>
            <nextchange>
                <endperiod>1538341200</endperiod>
                <tchange>42</tchange>
            </nextchange>
            <summeractive>1</summeractive>
            <holidayactive>0</holidayactive>
        </hkr>
    </device>
</devicelist>"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import eq_, assert_true, assert_false
from mock import MagicMock

from pyfritzhome import (FritzhomeDevice, Fritzhome)

from .elements import (device_list_xml, device_list_battery_ok_xml,
                       device_list_battery_low_xml, device_not_present_xml,
                       device_no_devicelock_element_xml,
                       device_with_umlaut_in_name_xml,
                       device_hkr_fw_03_50_xml, device_hkr_fw_03_54_xml,
                       device_hkr_no_temp_values_xml, device_alert_on_xml,
                       device_alert_off_xml, device_alert_no_alertstate_xml,
                       device_hkr_state_on_xml, device_hkr_state_off_xml,
                       device_hkr_state_eco_xml, device_hkr_state_comfort_xml,
                       device_hkr_state_manual_xml, device_hkr_fritzos_7_xml)


def get_switch_test_device():
    mock = MagicMock()
    mock.side_effect = [
        device_list_xml,
        '1'
    ]

    fritz = Fritzhome('10.0.0.1', 'user', 'pass')
    fritz._request = mock
    element = fritz.get_device_element('08761 0000434')
    device = FritzhomeDevice(fritz=fritz, node=element)
    return device


class TestDevice(object):

    def setup(self):
        self.mock = MagicMock()
        self.fritz = Fritzhome('10.0.0.1', 'user', 'pass')
        self.fritz._request = self.mock

    def test_device_init(self):
        self.mock.side_effect = [
            device_list_xml,
        ]

        device = self.fritz.get_device_by_ain('08761 0000434')

        eq_(device.ain, '08761 0000434')
        eq_(device.fw_version, '03.33')
        assert_true(device.present)
        assert_true(device.has_switch)
        assert_true(device.has_temperature_sensor)
        assert_true(device.has_powermeter)

    def test_device_init_present_false(self):
        self.mock.side_effect = [
            device_not_present_xml,
        ]

        device = self.fritz.get_device_by_ain('11960 0089208')

        eq_(device.ain, '11960 0089208')
        assert_false(device.present)

    def test_device_init_no_devicelock_element(self):
        self.mock.side_effect = [
            device_no_devicelock_element_xml,
        ]

        device = self.fritz.get_device_by_ain('08761 0373130')

        eq_(device.ain, '08761 0373130')
        assert_true(device.present)

    def test_device_umlaut(self):
        self.mock.side_effect = [
            device_with_umlaut_in_name_xml,
        ]

        device = self.fritz.get_device_by_ain('08761 0373130')

        eq_(device.ain, '08761 0373130')
        eq_(device.name, u'äöü')

    def test_device_hkr_fw_03_50(self):
        self.mock.side_effect = [
            device_hkr_fw_03_50_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        assert_true(device.present)
        eq_(device.device_lock, None)
        eq_(device.lock, None)
        eq_(device.error_code, None)
        eq_(device.battery_low, None)

    def test_device_hkr_fw_03_54(self):
        self.mock.side_effect = [
            device_hkr_fw_03_54_xml,
        ]

        device = self.fritz.get_device_by_ain('23456')
        assert_true(device.present)

    def test_device_alert_on(self):
        self.mock.side_effect = [
            device_alert_on_xml,
        ]

        device = self.fritz.get_device_by_ain('05333 0077045-1')
        assert_true(device.present)
        eq_(device.alert_state, True)

    def test_device_alert_off(self):
        self.mock.side_effect = [
            device_alert_off_xml,
        ]

        device = self.fritz.get_device_by_ain('05333 0077045-2')
        assert_true(device.present)
        eq_(device.alert_state, False)

    def test_device_alert_no_alertstate(self):
        self.mock.side_effect = [
            device_alert_no_alertstate_xml,
        ]

        device = self.fritz.get_device_by_ain('05333 0077045-3')
        assert_true(device.present)
        eq_(device.alert_state, None)

    def test_device_update(self):
        self.mock.side_effect = [
            device_list_battery_ok_xml,
            device_list_battery_low_xml,
        ]

        device = self.fritz.get_device_by_ain('11959 0171328')
        assert_false(device.battery_low)
        device.update()
        assert_true(device.battery_low)

    def test_get_device_present(self):
        self.mock.side_effect = [
            '1',
            '0'
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        assert_true(device.get_present())
        assert_false(device.get_present())
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'getswitchpresent', 'sid': None})

    def test_get_switch_state(self):
        self.mock.side_effect = [
            '1',
            '0'
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        assert_true(device.get_switch_state())
        assert_false(device.get_switch_state())
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'getswitchstate', 'sid': None})

    def test_set_switch_state(self):
        device = get_switch_test_device()
        device._fritz._request = self.mock

        device.set_switch_state_on()
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'setswitchon', 'sid': None})

        device.set_switch_state_off()
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'setswitchoff', 'sid': None})

        device.set_switch_state_toggle()
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'setswitchtoggle', 'sid': None})

    def test_get_switch_power(self):
        self.mock.side_effect = [
            '18000',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_switch_power(), 18000)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'getswitchpower', 'sid': None})

    def test_get_switch_energy(self):
        self.mock.side_effect = [
            '2000',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_switch_energy(), 2000)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'getswitchenergy', 'sid': None})

    def test_get_temperature(self):
        self.mock.side_effect = [
            '245',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_temperature(), 24.5)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'gettemperature', 'sid': None})

    def test_get_target_temperature(self):
        self.mock.side_effect = [
            '38',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_target_temperature(), 19.0)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'gethkrtsoll', 'sid': None})

    def test_get_eco_temperature(self):
        self.mock.side_effect = [
            '40',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_eco_temperature(), 20.0)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'gethkrabsenk', 'sid': None})

    def test_get_comfort_temperature(self):
        self.mock.side_effect = [
            '41',
        ]

        device = get_switch_test_device()
        device._fritz._request = self.mock

        eq_(device.get_comfort_temperature(), 20.5)
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'08761 0000434', 'switchcmd':
             'gethkrkomfort', 'sid': None})

    def test_hkr_without_temperature_values(self):
        self.mock.side_effect = [
            device_hkr_no_temp_values_xml,
        ]

        element = self.fritz.get_device_element('11960 0071472')
        device = FritzhomeDevice(node=element)

        eq_(device.ain, '11960 0071472')
        eq_(device.offset, None)
        eq_(device.temperature, None)

    def test_hkr_get_state_on(self):
        self.mock.side_effect = [
            device_hkr_state_on_xml,
            device_hkr_state_on_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.get_hkr_state(), 'on')

    def test_hkr_get_state_off(self):
        self.mock.side_effect = [
            device_hkr_state_off_xml,
            device_hkr_state_off_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.get_hkr_state(), 'off')

    def test_hkr_get_state_eco(self):
        self.mock.side_effect = [
            device_hkr_state_eco_xml,
            device_hkr_state_eco_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.get_hkr_state(), 'eco')

    def test_hkr_get_state_comfort(self):
        self.mock.side_effect = [
            device_hkr_state_comfort_xml,
            device_hkr_state_comfort_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.get_hkr_state(), 'comfort')

    def test_hkr_get_state_manual(self):
        self.mock.side_effect = [
            device_hkr_state_manual_xml,
            device_hkr_state_manual_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.get_hkr_state(), 'manual')

    def test_hkr_set_state_on(self):
        self.mock.side_effect = [
            device_hkr_state_manual_xml,
            device_hkr_state_manual_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')

        device.set_hkr_state('on')
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'12345', 'switchcmd':
             'sethkrtsoll', 'param': 254, 'sid': None})

    def test_hkr_set_state_off(self):
        self.mock.side_effect = [
            device_hkr_state_manual_xml,
            device_hkr_state_manual_xml,
        ]

        device = self.fritz.get_device_by_ain('12345')

        device.set_hkr_state('off')
        device._fritz._request.assert_called_with(
            'http://10.0.0.1/webservices/homeautoswitch.lua',
            {'ain': u'12345', 'switchcmd':
             'sethkrtsoll', 'param': 253, 'sid': None})

    def test_hkr_battery_level(self):
        self.mock.side_effect = [
            device_hkr_fritzos_7_xml
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.battery_level, 70)

    def test_hkr_window_open(self):
        self.mock.side_effect = [
            device_hkr_fritzos_7_xml
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.window_open, False)

    def test_hkr_summer_active(self):
        self.mock.side_effect = [
            device_hkr_fritzos_7_xml
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.summer_active, True)

    def test_hkr_holiday_active(self):
        self.mock.side_effect = [
            device_hkr_fritzos_7_xml
        ]

        device = self.fritz.get_device_by_ain('12345')
        eq_(device.holiday_active, False)
